SystemOrganization addCategory: #'AST-Interpreter'!
SystemOrganization addCategory: #'AST-Interpreter-Test'!

TestCase subclass: #ASTInterpreterTest
	instanceVariableNames: 'interpreter context'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AST-Interpreter-Test'!

!ASTInterpreterTest class methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/9/2011 19:56'!
testMethodArguments: anObject
	^ [ anObject ] value! !

!ASTInterpreterTest methodsFor: 'helper' stamp: 'CamilloBruni 12/5/2011 19:01'!
compile: aString
	| ast |
	ast := RBParser parseExpression: aString.
	ast annotateInClass: ProtoObject.
	^ ast! !

!ASTInterpreterTest methodsFor: 'helper' stamp: 'CamilloBruni 12/12/2011 15:03'!
interpret: aString
	| result |
	
	"context homeContext: thisContext".
	interpreter resetContext: context.
	result := interpreter interpret: (self compile: aString).
	
	self assert: interpreter context = context.
	^ result! !

!ASTInterpreterTest methodsFor: 'helper' stamp: 'CamilloBruni 12/9/2011 22:21'!
setUp
	"for now we use an ast cache to speed up the ast lookup"
	ASTCache initialize.
	interpreter := ASTInterpreter new.
	context := AIRootContext new.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/9/2011 19:53'!
testBasicCode
	self assert: (self interpret: '[ true ifTrue: [ nil ] ifFalse: [ 1 ]] value') = nil.
	self assert: (self interpret: '[ false ifTrue: [ nil ] ifFalse: [ 1 ]] value') = 1.
	
	self assert: (self interpret: '[ |a| a :=1. [ a < 10 ] whileTrue: [ a := a + 1]. a] value') = 10.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 10/5/2011 17:39'!
testBinarySend
	self assert: (self interpret: '1 + 2') = 3.
	self assert: (self interpret: '1 < 2') = true.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/5/2011 18:26'!
testBlock
	self assert: ((self interpret: '[ 1 ]') isKindOf: ASTBlockClosure).
	
	self assert: (self interpret: '[ 1 ] value') = 1.
	self assert: (self interpret: '[ 1 + 2 ] value') = 3.
	
	self assert: (self interpret: 'true ifTrue: [ 1 ] ifFalse: [ 0 ]') = 1.
	self assert: (self interpret: 'false ifTrue: [ 1 ] ifFalse: [ 0 ]') = 0.
	! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/5/2011 18:10'!
testBlockArgument

	self assert: (self interpret: '[ :a| a ] value: 1') = 1.
	self assert: (self interpret: '[ :a| a + 3 ] value: 3 ') = 6.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/8/2011 15:31'!
testBlockVar

	self assert: (self interpret: '[ |a| a := 1. a ] value') = 1.
	self assert: (self interpret: '[ |a| a := 1 + 2. a + 3 ] value') = 6.
	
	self assert: (self interpret: '[ |a| [ a := 1 ] value ] value') = 1.
	self assert: (self interpret: '[ |a| a := 1. [ a := 2 ] value ] value') = 2.
	
	self assert: (self interpret: '[ |a| a := 1. [ a := a + 1 ] value ] value') = 2.
	
	self assert: (self interpret: '[ :b ||a| a := 1. [ :c| a := a + 1 + c ] value: b ] value: 3') = 5.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/5/2011 09:56'!
testConstant
	self assert: (self interpret: '1') = 1.
	self assert: (self interpret: 'nil') = nil.
	self assert: (self interpret: 'false') = false.
	self assert: (self interpret: '''asdf''') = 'asdf'.
	self assert: (self interpret: '#symbol') = #symbol.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/12/2011 15:00'!
testDNU
	| error |
	
	[ self interpret: '1 aMessageNotUnderstoodBySmallInteger' ]
		on: InterpretationError
		do: [ :err| error := err ].
	
	self assert: error cause message selector = #aMessageNotUnderstoodBySmallInteger.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/12/2011 16:35'!
testException
	[ (self interpret: 'Error signal') ]
		on: InterpretationError
		do: [ :err| ^ self ].
	self fail.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/12/2011 16:33'!
testExceptionHandling

	self assert: (self interpret: '[ ] on: Error do: [ :err| false ].') = nil.
	self assert: (self interpret: '[ ] on: Error do: [ :err| false ]. true') = true.
	self assert: (self interpret: '[ ] on: Error do: [ :err| ^ false ]. true') = true.
	
	self assert: (self interpret: '[ Error signal ] on: Error do: [ :err| false ]') = false.
	self assert: (self interpret: '[ Error signal ] on: Error do: [ :err| false ]. true') = true.
	self assert: (self interpret: '[ Error signal ] on: Error do: [ :err| ^ false ]. true') = false.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/5/2011 14:56'!
testGlobals
	self assert: (self interpret: 'Array') = Array! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/8/2011 18:19'!
testInstanceCreation
	self assert: ((self interpret: 'Array new') isKindOf: Array).
	self assert: ((self interpret: 'Error new') isKindOf: Error).
	self assert: ((self interpret: 'MessageNotUnderstood new') isKindOf: MessageNotUnderstood)! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/9/2011 19:55'!
testMethodArguments
	
	self assert: (self interpret: 'ASTInterpreterTest testMethodArguments: 1') = 1.
	! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/8/2011 18:21'!
testNonLocalReturn

	self assert: (self interpret: 'false ifTrue: [ ^ 1 ]. ^ 1 + 1') = 2.
	self assert: (self interpret: 'true ifTrue: [ ^ 1 ]. ^ 1 + 1') = 1.
	
	self assert: (self interpret: 'false ifTrue: [ ^ 1 ] ifFalse: [^ 2]. 1 + 1 + 1') = 2.! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/8/2011 00:21'!
testPrimitive
	self assert: (self interpret: '1 + 1') = 2. "simple"
	self assert: (self interpret: '1 + 1.5') = 2.5. "with fallback"! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/9/2011 20:54'!
testSend
	self assert: (self interpret: '#(1 2) at: 1') = 1.
	self assert: (self interpret: '1 + 2') = 3.
	"this is for future use :)"
	"self assert: (self interpret: '123 asString') = '123'."! !

!ASTInterpreterTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/5/2011 11:24'!
testUnarySend
	self assert: (self interpret: '1 asInteger') = 1.
	self assert: (self interpret: '1 class') = SmallInteger.! !

!RBInstanceBinding methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/5/2011 15:18'!
accept: aVisitor assign: aValue
	^ aVisitor write: aValue at: index named: name! !

!RBInstanceBinding methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/5/2011 15:22'!
readWith: aVisitor
	^ aVisitor readInstVarAt: index named: name! !

!RBParser methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 8/23/2011 16:12'!
selfNodeClass
	^ RBSelfNode! !

!RBParser methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 8/23/2011 16:12'!
superNodeClass
	^ RBSuperNode! !

!RBParser methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 8/30/2011 17:03'!
thisContextNodeClass
	^ RBThisContextNode! !

!RBTemporaryBinding methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/5/2011 18:36'!
accept: anInterpeter assign: value
	self flag: ' for now we rely on the temporaries being dicts'.
	^ anInterpeter write: value temporaryAt: -1 named: node name! !

!RBTemporaryBinding methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/5/2011 18:46'!
readWith: anInterpreter
	self flag: ' for now we rely on the temporaries being dicts'.
	^ anInterpreter readTemporaryAt: -1 named: node name! !

!RBBlockNode methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/8/2011 15:17'!
temporaryNames
	^ self body temporaryNames! !

!RBMethodNode methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/8/2011 16:19'!
temporaryNames
	^ self body temporaryNames! !

!RBProgramNodeVisitor methodsFor: '*AST-interpreter' stamp: 'CamilloBruni 12/5/2011 19:21'!
acceptSelfNode: aSelfNode
	^ self acceptVariableNode: aSelfNode! !

!RBProgramNodeVisitor methodsFor: '*AST-interpreter' stamp: 'CamilloBruni 12/5/2011 19:28'!
acceptSuperNode: aSuperNode
	^ self acceptVariableNode: aSuperNode! !

!RBProgramNodeVisitor methodsFor: '*AST-interpreter' stamp: 'CamilloBruni 12/8/2011 15:27'!
acceptThisContextNode: aThisContextNode
	^ self acceptVariableNode: aThisContextNode! !

MessageNotUnderstood subclass: #RecursiveMessageNotUnderstood
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AST-Interpreter'!

!RBVariableBinding methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/5/2011 15:05'!
isArgument
	^ self isArgumentBinding! !

!RBVariableBinding methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/5/2011 15:05'!
isContext
	^ self isContextBinding! !

!RBVariableBinding methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/5/2011 15:05'!
isInstance
	^ self isInstanceBinding! !

!RBVariableBinding methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/5/2011 15:06'!
isLiteral
	^ self isLiteralBinding! !

!RBVariableBinding methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/5/2011 15:06'!
isSelf
	^ self isSelf! !

!RBVariableBinding methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/5/2011 15:06'!
isSuper
	^ self isSuperBinding! !

!RBVariableBinding methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/5/2011 15:06'!
isTemp
	^ self isTemporaryBinding! !

!RBVariableBinding methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/5/2011 15:06'!
isUndeclared
	^ self isUndeclaredBinding! !

!CompiledMethod methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/5/2011 16:12'!
accept: visitor on: receiver message: aMessage
	self isPrimitive ifTrue: [ 
			^ visitor 
				invokePrimitiveMethod: self 
				on: receiver 
				message: aMessage ].
			
	self isReflective ifTrue: [ 
		^ visitor 
			invokeReflectiveMethod: self
			on: receiver 
			message: aMessage].
		
	^ visitor 
		invokeMethod: self 
		on: receiver 
		message: aMessage! !

!CompiledMethod methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/9/2011 13:59'!
ast
	^ ASTCache at: self! !

!CompiledMethod methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/5/2011 14:14'!
code
	^ self ast! !

!CompiledMethod methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/5/2011 11:08'!
isPrimitive
	^ self primitive ~= 0! !

!CompiledMethod methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/5/2011 16:14'!
isReflective
	^ (self pragmaAt: #reflective:) isNil not! !

!RBArgumentBinding methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/9/2011 20:00'!
readWith: anInterpreter
	| index | 
	index := self scope node argumentNames indexOf: node name.
	^ anInterpreter readArgumentAt: index node: node! !

RBVariableNode subclass: #RBClassReference
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AST-Interpreter'!

RBVariableNode subclass: #RBSelfNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AST-Interpreter'!

!RBSelfNode methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/9/2011 14:36'!
acceptVisitor: aProgramNodeVisitor 
	^ aProgramNodeVisitor acceptSelfNode: self! !

RBVariableNode subclass: #RBSuperNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AST-Interpreter'!

!RBSuperNode methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 8/30/2011 16:54'!
acceptVisitor: aProgramNodeVisitor 
	^aProgramNodeVisitor acceptSuperNode: self! !

RBVariableNode subclass: #RBThisContextNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AST-Interpreter'!

!RBThisContextNode methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 8/30/2011 16:54'!
acceptVisitor: aProgramNodeVisitor 
	^aProgramNodeVisitor acceptThisContextNode: self! !

!RBVariableNode class methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 8/30/2011 17:22'!
identifierToken: anIdentifierToken 
	anIdentifierToken value = 'self'
		ifTrue: [ ^ RBSelfNode withToken: anIdentifierToken ].
	anIdentifierToken value = 'thisContext'
		ifTrue: [ ^ RBThisContextNode withToken: anIdentifierToken ].
	anIdentifierToken value = 'super'
		ifTrue: [ ^ RBSuperNode withToken: anIdentifierToken ].
	^ self withToken: anIdentifierToken! !

!RBVariableNode class methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/5/2011 19:19'!
withToken: anIdentifierToken
	^(self new)
		identifierToken: anIdentifierToken;
		yourself! !

!RBVariableNode methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/5/2011 15:04'!
binding
	^ self variableBinding! !

!RBLiteralBinding methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/5/2011 16:02'!
readWith: anInterpreter
	self flag: 'maybe introduce indirections over the interpreter'.
	^ self value! !

!RBLiteralBinding methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/5/2011 15:11'!
value
	^ self binding value! !

Object subclass: #AIContext
	instanceVariableNames: 'closure temporaries arguments outerContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AST-Interpreter'!

AIContext subclass: #AIBlockContext
	instanceVariableNames: 'homeContext exceptionHandler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AST-Interpreter'!

!AIBlockContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/9/2011 20:52'!
exceptionHandler
	^ exceptionHandler! !

!AIBlockContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/9/2011 20:52'!
exceptionHandler: anExceptionHandler
	exceptionHandler := anExceptionHandler! !

!AIBlockContext methodsFor: 'testing' stamp: 'CamilloBruni 12/12/2011 14:19'!
hasExceptionHandler
	^ exceptionHandler isNil not! !

!AIBlockContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/9/2011 14:57'!
homeContext

	^ homeContext! !

!AIBlockContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/9/2011 14:57'!
homeContext: aContext

	homeContext := aContext! !

!AIBlockContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/9/2011 20:44'!
methodClass
	self flag: 'use the same closures everywhere..'.
	^ self outerContext methodClass! !

!AIBlockContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/9/2011 15:05'!
receiver
	^ self homeContext receiver! !

!AIBlockContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/12/2011 14:33'!
returnContext
	self flag: 'TODO: not sure if this is correct...'.
	^ self homeContext returnContext! !

AIBlockContext subclass: #AIRootContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AST-Interpreter'!

!AIRootContext methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/9/2011 22:20'!
handleSignal: exception
	<reflective: #rootContextHandleSignal:message:>
	InterpretationError signal: exception! !

!AIRootContext methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/12/2011 14:19'!
hasExceptionHandler
	^ true! !

!AIRootContext methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/12/2011 15:34'!
outerContext
	^ outerContext ifNil: [ outerContext := AIRootContext new ].! !

!AIRootContext methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/12/2011 15:11'!
receiver
	^ self! !

!AIRootContext methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/12/2011 15:56'!
resume: aValue
	"<reflective: #escape:message:>"
	^ super resume: aValue! !

!AIRootContext methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/12/2011 15:32'!
returnContext
	^ self outerContext! !

!AIContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/5/2011 17:35'!
arguments
	^ arguments! !

!AIContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/5/2011 17:35'!
arguments: aCollection
	arguments := aCollection! !

!AIContext methodsFor: 'exception-handling' stamp: 'CamilloBruni 12/12/2011 14:42'!
cannotReturn: result to: homeContext
	"The receiver tried to return result to homeContext that no longer exists."

	^ BlockCannotReturn new
		result: result;
		deadHome: homeContext;
		signal! !

!AIContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/5/2011 17:33'!
closure

	^ closure! !

!AIContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/5/2011 17:33'!
closure: anObject
	
	closure := anObject! !

!AIContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/9/2011 20:01'!
code
	^ self closure code! !

!AIContext methodsFor: 'st-compatibility' stamp: 'CamilloBruni 12/8/2011 14:45'!
contextTag
	^ self! !

!AIContext methodsFor: 'exception-handling' stamp: 'CamilloBruni 12/9/2011 16:31'!
findContextSuchThat: testBlock
	"Search self and my sender chain for first one that satisfies testBlock.  Return nil if none satisfy"

	| ctxt |
	
	ctxt := self.
	[ctxt isNil] whileFalse: [
		(testBlock value: ctxt) 
			ifTrue: [^ ctxt].
		ctxt := ctxt outerContext ].

	^ nil! !

!AIContext methodsFor: 'exception-handling' stamp: 'CamilloBruni 12/12/2011 14:19'!
findNextHandlerContextStarting
	"Return the next handler marked context, returning nil if there is none.  Search starts with self and proceeds up to nil."
	
	^ self findContextSuchThat: [ :ctxt| ctxt hasExceptionHandler ]! !

!AIContext methodsFor: 'exception-handling' stamp: 'CamilloBruni 12/12/2011 14:49'!
handleSignal: exception
	"Sent to handler (on:do:) contexts only.  If my exception class (first arg) handles exception then execute my handle block (second arg), otherwise forward this message to the next handler context.  If none left, execute exception's defaultAction (see nil>>handleSignal:)."

	| val |
	
	(self exceptionHandler handles: exception)
		ifFalse: [ ^ self nextHandlerContext handleSignal: exception].

	exception privHandlerContext: self contextTag.
	
	"self tempAt: 3 put: false. " "disable self while executing handle block"
	
	"val := [(self tempAt: 2) cull: exception]
		ensure: [self tempAt: 3 put: true]."
	val := self exceptionHandler handle: exception.
	
	self return: val.  "return from self if not otherwise directed in handle block"
! !

!AIContext methodsFor: 'testing' stamp: 'CamilloBruni 12/12/2011 14:20'!
hasExceptionHandler
	^ false! !

!AIContext methodsFor: 'initialize-release' stamp: 'CamilloBruni 12/5/2011 18:08'!
initialize
	self flag: 'for now the easies solution'.
	temporaries := SmallDictionary new! !

!AIContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/12/2011 14:57'!
isDead
	self flag: 'TODO:'.
	^ false! !

!AIContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/5/2011 19:36'!
methodClass
	^ closure methodClass! !

!AIContext methodsFor: 'exception-handling' stamp: 'CamilloBruni 12/9/2011 16:27'!
nextHandlerContext

	^ self outerContext findNextHandlerContextStarting! !

!AIContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/5/2011 17:33'!
outerContext

	^ outerContext! !

!AIContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/5/2011 17:33'!
outerContext: anObject
	
	outerContext := anObject! !

!AIContext methodsFor: 'continuation' stamp: 'CamilloBruni 12/12/2011 16:07'!
resume: value
	"Unwind thisContext to self and resume with value as result of last send.  Execute unwind blocks when unwinding.  ASSUMES self is a sender of thisContext"

	| ctxt unwindBlock |
	self isDead 
		ifTrue: [self cannotReturn: value to: self].
		
"	ctxt := thisContext.
	[	ctxt := ctxt findNextUnwindContextUpTo: self.
		ctxt isNil
	] whileFalse: [
		(ctxt tempAt: 2) ifNil:[
			ctxt tempAt: 2 put: true.
			unwindBlock := ctxt tempAt: 1.
			thisContext terminateTo: ctxt.
			unwindBlock value].
	]."

	thisContext terminateTo: self value: value.
! !

!AIContext methodsFor: 'continuation' stamp: 'CamilloBruni 12/12/2011 16:33'!
return: value
	"Unwind thisContext to self and return value to self's sender.  Execute any unwind blocks while unwinding.  ASSUMES self is a sender of thisContext"

	self returnContext 
		ifNil: [self cannotReturn: value to: self returnContext].
		
	self resume: value! !

!AIContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/8/2011 17:52'!
tempNamed: aName
	self temporaries 
		at: aName 
		ifPresent: [ :value| ^ value].
	
	self outerContext 
		ifNil: [ Error signal: 'Could not find temp named "', aName , '"'].
		
	^ self outerContext tempNamed: aName.! !

!AIContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/8/2011 17:52'!
tempNamed: aName put: aValue
	(self temporaries includesKey: aName)
		ifTrue: [ ^ self temporaries at: aName put: aValue ].
	
	self outerContext 
		ifNil: [ Error signal: 'Could not find temp named "', aName , '"'].
	
	^ self outerContext tempNamed: aName put: aValue
		! !

!AIContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/5/2011 18:12'!
temporaries
	^ temporaries ifNil: [ temporaries := SmallDictionary new]! !

!AIContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/8/2011 15:23'!
temporaries: aNameCollection
	aNameCollection do: [ :aVarName|
		self temporaries at: aVarName put: nil ].! !

!AIContext methodsFor: 'continuation' stamp: 'CamilloBruni 12/12/2011 16:14'!
terminateTo: previousContext value: aReturnValue
	"Terminate all the Contexts between me and previousContext, if previousContext is on my Context stack. Make previousContext my sender."

	<reflective: #contextTerminateTo:message:>
	
	PrimitiveFailed signal.
	! !

AIContext subclass: #AIMethodContext
	instanceVariableNames: 'receiver'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AST-Interpreter'!

!AIMethodContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/9/2011 20:04'!
homeContext
	^ self returnContext! !

!AIMethodContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/5/2011 17:33'!
receiver

	^ receiver! !

!AIMethodContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/5/2011 17:33'!
receiver: anObject
	
	receiver := anObject! !

!AIMethodContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/5/2011 20:05'!
returnContext
	^ self outerContext! !

!AIMethodContext methodsFor: 'accessing' stamp: 'CamilloBruni 12/12/2011 15:51'!
returnContext: aContext
	^ self outerContext: aContext! !

Object subclass: #ASTCache
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AST-Interpreter'!

ASTCache class
	instanceVariableNames: 'cache'!

ASTCache class
	instanceVariableNames: 'cache'!

!ASTCache class methodsFor: 'accessing' stamp: 'CamilloBruni 12/9/2011 14:01'!
at: aCompiledMethod
	^ cache 
		at: aCompiledMethod 
		ifAbsentPut: [ 
			aCompiledMethod parseTree annotateInClass: aCompiledMethod methodClass ]! !

!ASTCache class methodsFor: 'class initialization' stamp: 'CamilloBruni 12/9/2011 14:00'!
initialize
	cache := IdentityDictionary new.! !

Object subclass: #ASTClosure
	instanceVariableNames: 'code'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AST-Interpreter'!

ASTClosure subclass: #ASTBlockClosure
	instanceVariableNames: 'homeContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AST-Interpreter'!

!ASTBlockClosure methodsFor: 'visiting' stamp: 'CamilloBruni 10/5/2011 17:47'!
accept: visitor
	^ visitor visitBlockClosure: self! !

!ASTBlockClosure methodsFor: 'evaluating' stamp: 'CamilloBruni 12/12/2011 14:12'!
cull: anArg
	^ self numArgs = 0 
		ifTrue: [self value]
		ifFalse: [self value: anArg]
	! !

!ASTBlockClosure methodsFor: 'evaluating' stamp: 'CamilloBruni 12/12/2011 14:12'!
cull: firstArg cull: secondArg
	^ self numArgs < 2 
		ifTrue: [self cull: firstArg]
		ifFalse: [self value: firstArg value: secondArg]
	! !

!ASTBlockClosure methodsFor: 'evaluating' stamp: 'CamilloBruni 12/12/2011 14:12'!
cull: firstArg cull: secondArg cull: thirdArg
	^ self numArgs < 3 
		ifTrue: [self cull: firstArg cull: secondArg]
		ifFalse: [self value: firstArg value: secondArg value: thirdArg]
	! !

!ASTBlockClosure methodsFor: 'evaluating' stamp: 'CamilloBruni 12/12/2011 14:12'!
cull: firstArg cull: secondArg cull: thirdArg cull: fourthArg
	"Execute the receiver with four or less arguments. Check cull:cull: for examples"
	
	^ self numArgs < 4 
		ifTrue: [self cull: firstArg cull: secondArg cull: thirdArg]
		ifFalse: [self value: firstArg value: secondArg value: thirdArg value: fourthArg]
	! !

!ASTBlockClosure methodsFor: 'accessing' stamp: 'CamilloBruni 12/9/2011 14:53'!
homeContext

	^ homeContext! !

!ASTBlockClosure methodsFor: 'accessing' stamp: 'CamilloBruni 12/9/2011 14:53'!
homeContext: aContext

	homeContext := aContext! !

!ASTBlockClosure methodsFor: 'evaluating' stamp: 'CamilloBruni 12/12/2011 16:13'!
on: exception do: aBlock
	<reflective: #blockClosureOnDo:message:>
	PrimitiveFailed signal! !

!ASTBlockClosure methodsFor: 'accessing' stamp: 'CamilloBruni 12/9/2011 14:53'!
returnContext

	^ homeContext returnContext! !

!ASTBlockClosure methodsFor: 'evaluating' stamp: 'CamilloBruni 12/5/2011 16:16'!
value
	<reflective: #blockClosureValue:message:>
	^ self valueWithArguments: (Array new: 0)! !

!ASTBlockClosure methodsFor: 'evaluating' stamp: 'CamilloBruni 12/5/2011 16:13'!
value: anArg
	<reflective: #blockClosureValue:message:>
	^ self valueWithArguments: (Array with: anArg)! !

!ASTBlockClosure methodsFor: 'evaluating' stamp: 'CamilloBruni 12/5/2011 16:13'!
value: anArg value: otherArg
	<reflective: #blockClosureValue:message:>
	^ self valueWithArguments: (Array with: anArg with: otherArg)! !

!ASTBlockClosure methodsFor: 'evaluating' stamp: 'CamilloBruni 12/5/2011 16:13'!
value: anArg value: otherArg value: anotherArg
	<reflective: #blockClosureValue:message:>
	^ self valueWithArguments: (Array with: anArg with: otherArg with: anotherArg)! !

!ASTBlockClosure methodsFor: 'evaluating' stamp: 'CamilloBruni 12/5/2011 16:13'!
value: anArg value: otherArg value: anotherArg value: yetAnotherArg
	<reflective: #blockClosureValue:message:>
	^ self valueWithArguments: (Array with: anArg with: otherArg with: anotherArg with: yetAnotherArg)! !

!ASTBlockClosure methodsFor: 'evaluating' stamp: 'CamilloBruni 12/5/2011 16:14'!
valueWithArguments: arguments
	<reflective: #blockClosureValueWithArguments:message:>
	PrimitiveFailed signal! !

!ASTBlockClosure methodsFor: 'controlling' stamp: 'CamilloBruni 10/5/2011 17:50'!
whileFalse
	self value ifFalse: [ self whileFalse ]! !

!ASTBlockClosure methodsFor: 'controlling' stamp: 'CamilloBruni 10/5/2011 17:51'!
whileFalse: aBlock
	^ self value 
		ifFalse: [ 
			aBlock value. 
			self whileFalse: aBlock ]! !

!ASTBlockClosure methodsFor: 'controlling' stamp: 'CamilloBruni 10/5/2011 17:50'!
whileTrue
	self value ifTrue: [ self whileTrue ]! !

!ASTBlockClosure methodsFor: 'controlling' stamp: 'CamilloBruni 10/5/2011 17:50'!
whileTrue: aBlock
	^ self value
		ifTrue: [ 
			aBlock value. 
			self whileTrue: aBlock ]! !

!ASTClosure methodsFor: 'accessing' stamp: 'CamilloBruni 10/5/2011 17:47'!
code

	^ code! !

!ASTClosure methodsFor: 'accessing' stamp: 'CamilloBruni 10/5/2011 17:47'!
code: anObject
	
	code := anObject! !

!ASTClosure methodsFor: 'accessing' stamp: 'CamilloBruni 12/12/2011 14:15'!
numArgs
	^ self code arguments size! !

!ASTClosure methodsFor: 'evaluating' stamp: 'CamilloBruni 10/5/2011 17:47'!
valueWithPossibleArgs: anArray
	| numArgs |
	numArgs := self numArgs.
	numArgs = 0
		ifTrue: [ ^ self value ].
	numArgs = anArray size
		ifTrue: [ ^ self valueWithArguments: anArray ].
	numArgs > anArray size
		ifTrue: [ ^ self valueWithArguments: anArray , (Array new: numArgs - anArray size) ].
	^ self valueWithArguments: (anArray copyFrom: 1 to: numArgs)! !

ASTClosure subclass: #ASTMethodClosure
	instanceVariableNames: 'host selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AST-Interpreter'!

!ASTMethodClosure methodsFor: 'visiting' stamp: 'CamilloBruni 10/5/2011 17:47'!
accept: visitor
	^ visitor visitMethodClosure: self! !

!ASTMethodClosure methodsFor: 'accessing' stamp: 'CamilloBruni 10/5/2011 17:48'!
host

	^ host! !

!ASTMethodClosure methodsFor: 'accessing' stamp: 'CamilloBruni 10/5/2011 17:48'!
host: anObject
	
	host := anObject! !

!ASTMethodClosure methodsFor: 'accessing' stamp: 'CamilloBruni 10/5/2011 17:48'!
selector

	^ selector! !

!ASTMethodClosure methodsFor: 'accessing' stamp: 'CamilloBruni 10/5/2011 17:48'!
selector: anObject
	
	selector := anObject! !

Object subclass: #ASTInterpreter
	instanceVariableNames: 'context gotoContext returnBlock exceptionBlock primitiveFailed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AST-Interpreter'!

!ASTInterpreter methodsFor: 'visiting' stamp: 'CamilloBruni 12/5/2011 15:17'!
acceptAssignmentNode: anAssignmentNode
	| value variable |
	value := anAssignmentNode value acceptVisitor: self.
	variable := anAssignmentNode variable.
	variable binding accept: self assign: value. 
	^ value! !

!ASTInterpreter methodsFor: 'visiting' stamp: 'CamilloBruni 12/9/2011 14:52'!
acceptBlockNode: aBlockNode
	^ ASTBlockClosure new
		homeContext: self context;
		code: aBlockNode! !

!ASTInterpreter methodsFor: 'visiting' stamp: 'CamilloBruni 12/5/2011 19:52'!
acceptCascadeNode: aCascadeNode
	| receiver lastResult |
	receiver := aCascadeNode receiver acceptVisitor: self.
	aCascadeNode messages do: [ :aMessageNode|
		lastResult := self acceptMessageNode: aMessageNode receiver: receiver.
		self ifSkip: [ ^ lastResult]].
	
	^ lastResult! !

!ASTInterpreter methodsFor: 'visiting' stamp: 'CamilloBruni 10/5/2011 17:38'!
acceptLiteralArrayNode: aNode
	^ aNode value! !

!ASTInterpreter methodsFor: 'visiting' stamp: 'CamilloBruni 10/5/2011 17:13'!
acceptLiteralNode: aLiteralNode
	^ aLiteralNode value! !

!ASTInterpreter methodsFor: 'visiting' stamp: 'CamilloBruni 12/8/2011 18:19'!
acceptMessageNode: aMessageNode
	| receiver |

	receiver := aMessageNode receiver acceptVisitor: self.
	
	^ self acceptMessageNode: aMessageNode receiver: receiver.! !

!ASTInterpreter methodsFor: 'visiting' stamp: 'CamilloBruni 12/7/2011 17:16'!
acceptMessageNode: aMessageNode receiver: receiver
	| arguments message lookUpClass|
	
	arguments := (aMessageNode arguments 
						collect: [ :argument| |arg|
							arg := (argument acceptVisitor: self).
							self ifSkip: [ ^ arg ].
							arg ]) asArray.
						
	message := Message selector: aMessageNode selector arguments: arguments.
	
	lookUpClass := aMessageNode isSuperSend 
						ifFalse: [ self classOf: receiver ]
						ifTrue: [ self currentMethodClass superclass ].
	
	^ self
		send: message
		to: receiver
		class: lookUpClass
		for: aMessageNode! !

!ASTInterpreter methodsFor: 'visiting' stamp: 'CamilloBruni 12/8/2011 00:10'!
acceptReturnNode: aReturnNode
	| returnValue |
	
	returnValue := aReturnNode value acceptVisitor: self.
	
	self ifSkip: [ ^ returnValue ].
	
	self assert: gotoContext == false.
	gotoContext := context returnContext.
	
	^ returnValue! !

!ASTInterpreter methodsFor: 'visiting' stamp: 'CamilloBruni 10/6/2011 15:51'!
acceptSelfNode: aSelf
	^ self currentSelf! !

!ASTInterpreter methodsFor: 'visiting' stamp: 'CamilloBruni 12/12/2011 13:07'!
acceptSequenceNode: aSequenceNode
	| lastResult |
	 
	aSequenceNode statements do: [ :statement|
		lastResult := statement acceptVisitor: self.
		self ifSkip: [ ^ lastResult ]].
	
	self updateGotoContext.
		
	^ lastResult! !

!ASTInterpreter methodsFor: 'visiting' stamp: 'CamilloBruni 12/5/2011 19:31'!
acceptSuperNode: aSelf
	^ self currentSelf! !

!ASTInterpreter methodsFor: 'visiting' stamp: 'CamilloBruni 12/5/2011 15:14'!
acceptThisContextNode: thisContextNode
	^ self context! !

!ASTInterpreter methodsFor: 'visiting' stamp: 'CamilloBruni 12/5/2011 15:24'!
acceptVariableNode: aVariableNode
	^ aVariableNode binding readWith: self ! !

!ASTInterpreter methodsFor: 'reflective' stamp: 'CamilloBruni 12/12/2011 15:52'!
blockClosureOnDo: aBlockClosure message: aMessage
	| block result returnContext |
	
	self flag: 'refactor together with blockClosureValue:message:'.
	
	block := aBlockClosure code.
	
	self assert: aMessage numArgs = 2.
	
	returnContext := context.
	self blockContextFor: aBlockClosure message: aMessage.
	context exceptionHandler: (ExceptionHandler forMessage: aMessage).
	
	result := self interpretBlock: block.
	
	context := returnContext.
	^ result! !

!ASTInterpreter methodsFor: 'reflective' stamp: 'CamilloBruni 12/12/2011 14:15'!
blockClosureValue: aBlockClosure message: aMessage
	| block result returnContext |
	
	block := aBlockClosure code.
	
	self assert: aBlockClosure numArgs = aMessage numArgs.
	block size = 0 ifTrue: [ ^ nil ].
	
	returnContext := context.
	self blockContextFor: aBlockClosure message: aMessage.
	
	result := self interpretBlock: block.
	
	context := returnContext.
	^ result! !

!ASTInterpreter methodsFor: 'reflective' stamp: 'CamilloBruni 12/5/2011 16:20'!
blockClosureValueWithArguments: aBlockClosure message: aMessage
	^ self blockclosureValue: aBlockClosure message: (aMessage at: 1)! !

!ASTInterpreter methodsFor: 'context' stamp: 'CamilloBruni 12/9/2011 14:59'!
blockContextFor: aBlockClosure message: aMessage
	| block size |
	
	block := aBlockClosure code.
	size := block allDefinedVariables size.
	
	"size = 0 
		ifTrue: [ ^ context := aBlockClosure returnContext ]."
	
	context := AIBlockContext new
		homeContext: aBlockClosure homeContext;
		outerContext: context;
		closure: aBlockClosure;
		arguments: aMessage arguments;
		temporaries: block temporaryNames.! !

!ASTInterpreter methodsFor: 'message-sending' stamp: 'CamilloBruni 10/5/2011 17:24'!
classOf: anObject
	^ anObject class! !

!ASTInterpreter methodsFor: 'interpretation' stamp: 'CamilloBruni 12/5/2011 18:38'!
compile: aString
	| ast |
	ast := RBParser parseExpression: aString.
	ast annotateInClass: ProtoObject.
	^ ast! !

!ASTInterpreter methodsFor: 'accessing' stamp: 'CamilloBruni 10/6/2011 15:53'!
context
	^ context! !

!ASTInterpreter methodsFor: 'accessing' stamp: 'CamilloBruni 12/5/2011 20:14'!
context: aContext
	context := aContext! !

!ASTInterpreter methodsFor: 'reflective' stamp: 'CamilloBruni 12/12/2011 16:17'!
contextTerminateTo: aContext message: aMessage
	
	gotoContext := aMessage arguments first.
	^ aMessage arguments second
	! !

!ASTInterpreter methodsFor: 'accessing' stamp: 'CamilloBruni 12/8/2011 00:23'!
currentMethodClass
	^ context methodClass! !

!ASTInterpreter methodsFor: 'accessing' stamp: 'CamilloBruni 12/8/2011 16:02'!
currentSelf
	^ self context receiver! !

!ASTInterpreter methodsFor: 'reflective' stamp: 'CamilloBruni 12/12/2011 14:53'!
escape: aClosure message: aMessage
	"avoid interpreting this message in the interpreter"
	self halt.! !

!ASTInterpreter methodsFor: 'interpretation' stamp: 'CamilloBruni 12/5/2011 20:43'!
ifSkip: aReturnBlock
	gotoContext == false ifTrue: [ ^ self ].
		
	aReturnBlock value.! !

!ASTInterpreter methodsFor: 'interpretation' stamp: 'CamilloBruni 10/5/2011 17:13'!
interpret: anASTNode
	^ anASTNode acceptVisitor: self.! !

!ASTInterpreter methodsFor: 'interpretation' stamp: 'CamilloBruni 12/5/2011 16:28'!
interpretBlock: block
	^ block body acceptVisitor: self! !

!ASTInterpreter methodsFor: 'interpretation' stamp: 'CamilloBruni 10/6/2011 15:46'!
interpretMethod: method
	^ method body acceptVisitor: self! !

!ASTInterpreter methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/5/2011 20:44'!
intialize
	super initialize.
	gotoContext := false.! !

!ASTInterpreter methodsFor: 'method-evaluation' stamp: 'CamilloBruni 12/12/2011 13:28'!
invokeMethod: aClosure on: receiver message: aMessage
	| result |
	
	self assert: aClosure numArgs = aMessage numArgs.
	aClosure size = 0 
		ifTrue: [ ^ receiver ].
	
	self
		methodContextFor: aClosure
		receiver: receiver
		message: aMessage.
	
	FileStream stderr 
		nextPutAll: (((self currentSelf class name, ':') padded: #right to: 20 with: $ ), aClosure printString); cr.
	"self halt."
	result := self interpretMethod: aClosure code.
	
	self isExplicitelyReturning 
		ifFalse: [ result := self currentSelf ].
		
	self popContext.
	
	^ result! !

!ASTInterpreter methodsFor: 'method-evaluation' stamp: 'CamilloBruni 12/8/2011 00:08'!
invokePrimitiveMethod: aCompiledMethod on: receiver message: aMessage
	| result |
	self flag: 'TODO: should track the tower of interpreters properly...'.
	primitiveFailed := false.
	result := receiver tryPrimitive: aCompiledMethod primitive withArgs: aMessage arguments.
	result = ContextPart primitiveFailToken ifFalse: [ ^ result ].
	"primitive failed"
	^ self
		invokeMethod: aCompiledMethod 
		on: receiver 
		message: aMessage! !

!ASTInterpreter methodsFor: 'method-evaluation' stamp: 'CamilloBruni 12/9/2011 22:22'!
invokeReflectiveMethod: aClosure on: receiver message: aMessage
	| annotation selector |
	
	annotation := aClosure pragmaAt: #reflective:.
	selector := annotation arguments first.
	
	(self respondsTo: selector) ifTrue: [
		^ self 
			perform: selector 
			withArguments: (Array with: receiver with: aMessage)].
		
	self halt.
	
	^ self invokeMethod: aClosure on: receiver message: aMessage! !

!ASTInterpreter methodsFor: 'interpretation' stamp: 'CamilloBruni 12/8/2011 18:27'!
isExplicitelyReturning
	^ gotoContext ~= false! !

!ASTInterpreter methodsFor: 'message-sending' stamp: 'CamilloBruni 10/5/2011 17:27'!
lookupSelector: selector in: class
	| currentClass |
	currentClass := class.
	[ currentClass isNil ] whileFalse: [ 
			currentClass methodDict at: selector ifPresent: [ :method | ^ method ].
			currentClass := currentClass superclass ].
	^ nil! !

!ASTInterpreter methodsFor: 'context' stamp: 'CamilloBruni 12/9/2011 14:28'!
methodContextFor: aCompiledMethod receiver: receiver message: aMessage
	context := AIMethodContext new
					receiver: receiver;
					outerContext: context;
					closure: aCompiledMethod;
					arguments: aMessage arguments;
					temporaries: aCompiledMethod code temporaryNames;
					yourself! !

!ASTInterpreter methodsFor: 'interpretation' stamp: 'CamilloBruni 12/8/2011 18:25'!
popContext
	context := context outerContext.
	
	self updateGotoContext.! !

!ASTInterpreter methodsFor: 'slot' stamp: 'CamilloBruni 12/5/2011 18:23'!
readArgumentAt: index named: aName
	^ self context arguments at: index! !

!ASTInterpreter methodsFor: 'slot' stamp: 'CamilloBruni 12/9/2011 20:07'!
readArgumentAt: index node: aVariableNode
	| variableContext |
	
	variableContext := self context.
	
	[ variableContext code = aVariableNode binding scope node ] 
		whileFalse: [
				variableContext := variableContext homeContext.
				self assert: variableContext isNil not ].
			
	^ variableContext arguments at: index! !

!ASTInterpreter methodsFor: 'slot' stamp: 'CamilloBruni 12/5/2011 15:29'!
readArgumentNamed: aName
	^ self context tempNamed: aName! !

!ASTInterpreter methodsFor: 'slot' stamp: 'CamilloBruni 12/5/2011 15:22'!
readInstVarAt: index named: name
	^ self currentSelf instVarAt: index! !

!ASTInterpreter methodsFor: 'slot' stamp: 'CamilloBruni 12/5/2011 18:46'!
readTemporaryAt: index named: name
	^ context tempNamed: name! !

!ASTInterpreter methodsFor: 'message-sending' stamp: 'CamilloBruni 10/5/2011 17:27'!
recursiveDoesNotUnderstand: aMessage on: receiver
	^ RecursiveMessageNotUnderstood new
		receiver: receiver;
		cls: receiver class;
		message: aMessage;
		signal! !

!ASTInterpreter methodsFor: 'accessing' stamp: 'CamilloBruni 12/5/2011 20:44'!
resetContext: aContext
	context := aContext.
	gotoContext :=  false.! !

!ASTInterpreter methodsFor: 'reflective' stamp: 'CamilloBruni 12/12/2011 15:54'!
rootContextHandleSignal: aRootContext message: aMessage
	^ InterpretationError signalFor: aMessage arguments first.! !

!ASTInterpreter methodsFor: 'message-sending' stamp: 'CamilloBruni 10/5/2011 17:24'!
send: aMessage to: receiver
	^ self
		send: aMessage
		to: receiver
		class: (self classOf: receiver)! !

!ASTInterpreter methodsFor: 'message-sending' stamp: 'CamilloBruni 12/5/2011 19:54'!
send: aMessage to: receiver class: class
	| method |
	
	method := self lookupSelector: aMessage selector in: class.
	method ifNil: [ ^ self sendDoesNotUnderstandFor: aMessage to: receiver ].
	
	^ method
		accept: self
		on: receiver
		message: aMessage! !

!ASTInterpreter methodsFor: 'message-sending' stamp: 'CamilloBruni 12/7/2011 18:40'!
send: aMessage to: receiver class: class for: aSend
	| method |
	
	method := self lookupSelector: aMessage selector in: class.
	method ifNil: [ ^ self sendDoesNotUnderstandFor: aMessage to: receiver ].
	
	^ method
		accept: self 
		on: receiver
		message: aMessage! !

!ASTInterpreter methodsFor: 'message-sending' stamp: 'CamilloBruni 12/8/2011 16:04'!
sendDoesNotUnderstandFor: aMessage to: receiver
	| dnu |
	
	aMessage selector = #doesNotUnderstand:
		ifTrue: [ 
			self halt.
			^ self recursiveDoesNotUnderstand: (aMessage at: 1) on: receiver ].
	dnu := Message selector: #doesNotUnderstand: argument: aMessage.
	^ self send: dnu to: receiver! !

!ASTInterpreter methodsFor: 'visiting' stamp: 'CamilloBruni 12/8/2011 18:25'!
updateGotoContext

	gotoContext = context
		ifTrue: [ gotoContext := false ].! !

!ASTInterpreter methodsFor: 'slot' stamp: 'CamilloBruni 12/5/2011 15:23'!
write: aValue at: index named: name
	^ self currentSelf instVarAt: index put: aValue! !

!ASTInterpreter methodsFor: 'slot' stamp: 'CamilloBruni 12/5/2011 18:45'!
write: aValue temporaryAt: index named: name
	^ context tempNamed: name put: aValue! !

Object subclass: #ExceptionHandler
	instanceVariableNames: 'block exception'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AST-Interpreter'!

!ExceptionHandler class methodsFor: 'instance-creation' stamp: 'CamilloBruni 12/11/2011 18:46'!
forMessage: aMessage
	^ self 
		on: aMessage arguments first
		do: aMessage arguments second! !

!ExceptionHandler class methodsFor: 'instance-creation' stamp: 'CamilloBruni 12/12/2011 14:07'!
on: anException do: aBlock
	^ self new
		exception: anException;
		block: aBlock;
		yourself! !

!ExceptionHandler methodsFor: 'accessing' stamp: 'CamilloBruni 12/8/2011 15:47'!
block

	^ block! !

!ExceptionHandler methodsFor: 'accessing' stamp: 'CamilloBruni 12/8/2011 15:48'!
block: aBlock
	
	block := aBlock! !

!ExceptionHandler methodsFor: 'accessing' stamp: 'CamilloBruni 12/12/2011 14:10'!
exception

	^ exception! !

!ExceptionHandler methodsFor: 'accessing' stamp: 'CamilloBruni 12/12/2011 14:09'!
exception: anException

	exception := anException! !

!ExceptionHandler methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/12/2011 14:09'!
handle: anError

	self flag: 'TODO:do the ensure magic here..'.
	^ self block cull: anError! !

!ExceptionHandler methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/12/2011 14:09'!
handles: anError

	^ self exception handles: anError! !

Error subclass: #InterpretationError
	instanceVariableNames: 'cause'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AST-Interpreter'!

!InterpretationError class methodsFor: 'signalling' stamp: 'CamilloBruni 12/9/2011 14:08'!
signalFor: aCauseError
	^ self new
		cause: aCauseError;
		signal! !

!InterpretationError methodsFor: 'accessing' stamp: 'CamilloBruni 12/9/2011 14:08'!
cause

	^ cause! !

!InterpretationError methodsFor: 'accessing' stamp: 'CamilloBruni 12/9/2011 14:08'!
cause: anObject
	
	cause := anObject! !

!InterpretationError methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 12/12/2011 14:29'!
isResumable
	^ true! !

!ContextPart methodsFor: '*AST-Interpreter' stamp: 'CamilloBruni 12/12/2011 14:33'!
returnContext
	^ self methodReturnContext! !
ASTCache initialize!
